#!/usr/bin/php
<?php
define('NAME', 'check_api_extreme_networks_ap.php');
define('DESCRIPTION', "Check parameters of an Extrem Networks access point connected to the API (ExtremeCloud IQ)");
define('VERSION', '1');

require_once '/usr/local/esia/plugins/lib/php/plugins.lib.php';
include_once '/usr/local/esia/plugins/lib/php/HTTP.class.php';
include_once '/usr/local/esia/plugins/lib/php/CacheFile.class.php';

// Require auth file DEFAULT_AUTH_FILE or param -A
// Format:
// username="<user>"
// password="<pwd>"

define('STR_STATUS', array('OK', 'WARNING', 'CRITICAL', 'UNKNOWN'));

define('API_URL', 'https://api.extremecloudiq.com/');
define('DEFAULT_AUTH_FILE', '/etc/esia/extreme_networks.conf');
define('DEFAULT_CACHE_DIR_PATH', '/tmp/esia/extreme_networks/');
define('DEFAULT_CURLOPT_HTTPHEADER', array(
	'content-type: application/json; charset=utf-8',
));

define('AUTH_TOKEN_VALIDITY_GAP', 600); // Seconds (1 min). Gap before token expiration (The token expiration value is given by the api. This gap is subtracted from the value)
define('HTTP_CONNECTION_TIMEOUT', 30); // The number of seconds to wait while trying to connect to the API
define('LOCK_FILE_DURATION', 5); // Minutes to let the lock cache file valid (must be lower than CACHE_DEVICES_VALIDITY). If the lock cache file is valid, another process cannot regenerate the devices data. When a process is regeneration the devices data it regenerate first a lock cache file with this duration.
define('CACHE_DEVICES_VALIDITY', 6); // Cache during in minutes for devices list
define('CACHE_DEVICE_PERF_VALIDITY', 5); // Cache during in minutes for device perfs (cpu/mem)
define('DEFAULT_TEST_INTERVAL', 300); // Seconds (5min)
define('DEFAULT_OLD_DATA_AGE', 120); // Minutes. Used to avoid unknown state for a request timeout on perf request (cpu/mem). Work with old data during this duration before going to unknown state

$ERRORS = ESIA\plugins\ERRORS;

$hostName = NULL;
$authFile = DEFAULT_AUTH_FILE;
$type = 'status';
$warning = NULL;
$critical = NULL;
$configMissState = 'w';
$allowedDataAge = DEFAULT_OLD_DATA_AGE;
$isJson = FALSE;
$isDebug = FALSE;
$help = NULL;
$arr = array(
	'H=s' => &$hostName, 'host=s' => &$hostName,
	'A=s' => &$authFile, 'authfile=s' => &$authFile,
	'T=s' => &$type, 'type=s' => &$type,
	'w=i' => &$warning, 'warning=i' => &$warning,
	'c=i' => &$critical, 'critical=i' => &$critical,
	's=s' => &$configMissState, 'status=s' => &$configMissState,
	'o=i' => &$allowedDataAge, 'old=i' => &$allowedDataAge,
	'J' => &$isJson, 'json' => &$isJson,
	'D' => &$isDebug, 'debug' => &$isDebug,
	'h' => &$help, 'debug' => &$help,
);

$optError = FALSE;
try
{
	ESIA\plugins\getOptions($arr);
}
catch(ESIA\plugins\EsiaBadParameterType $e)
{
	$optError = TRUE;
	echo $e->getMessage() . "\n";
}

if(isset($help) || $optError)
{
	$opt = array();
	$opt[] = array('H', 'host', 'Device host name');
	$opt[] = array('A', 'authfile', 'Auth file path for API login credentials (Format: username=\"<user>\" password=\"<pwd>\")');
	$opt[] = array('T', 'type', 'Type of test [cpu, mem, status, config, clients, inventory] (default: status)');
	$opt[] = array('w', 'warning', 'Warning threshold in percent');
	$opt[] = array('c', 'critical', 'Critical threshold in percent');
	$opt[] = array('s', 'status', 'Only for -T config. Status for configuration mismatch found (o => ok, w => warning, c => critical, u => unknown). Default warning');
	$opt[] = array('o', 'old', 'Number of minutes allowed for data age (cpu & mem only). Default: ' . DEFAULT_OLD_DATA_AGE);
	$opt[] = array('J', 'json', 'JSON output for inventory -T inventory required');
	$opt[] = array('D', 'debug', 'Debug mode');
	$opt[] = array('h', 'help', 'Print this help');

	ESIA\plugins\print_help(NAME, DESCRIPTION, $opt);
	exit($ERRORS['UNKNOWN']);
}

// Check if the lock file is valid (another script is refreshing the data). If not, re-create the lock file with new time stamp & duration.
// TRUE => lock file is regenerated and the data can be refreshed.
// FALSE => lock file has been created by another process (will refresh data) and data must not be refreshed by the current process.
function isLockFileReset($file)
{
	$lockCache = new ESIA\CacheFile\CacheFile($file);
	
	if($lockCache->isValid())
	{
		// Duration time not elapsed. Another process is refreshing data
		return FALSE;
	}

	// Set during in minutes
	$lockCache->setDuring(LOCK_FILE_DURATION);
	// Write data or the file will not be created
	$lockCache->writeData(json_encode('locked'));

	// Lock file time reset. Data can be refreshed
	return TRUE;
}

// Create the http client with connection initialized by token retrieved in cache file or new token request
function initHttpClient($authFile, $cacheFileToken, $isDebug, $lockFile=NULL)
{
	try
	{
		$httpClient = NULL;
		$token = '';
		// Get API token from cache or request new token if expired
		$cacheFile = new ESIA\CacheFile\CacheFile($cacheFileToken);
		if($cacheFile->isValid())
		{
			$cacheData = json_decode($cacheFile->getData(), TRUE);
			if(!empty($cacheData['access_token']))
			{
				$token = $cacheData['access_token'];
			}
		}

		// Request api new auth token if not found in cache or expired
		if(empty($token))
		{
			// Parse config file
			$authHead = parse_ini_file($authFile);
			if(!empty($authHead['username']) && !empty($authHead['password']))
			{
				// Authentication to get access token
				// Instance httpClient for auth
				$httpClientAuth = new ESIA\HTTP\HttpClient(API_URL);
				$httpClientAuth->open();
				$httpClientAuth->setoptions(array(
					CURLOPT_CONNECTTIMEOUT => HTTP_CONNECTION_TIMEOUT,
					CURLOPT_TIMEOUT => HTTP_CONNECTION_TIMEOUT,
					CURLOPT_HTTPHEADER => DEFAULT_CURLOPT_HTTPHEADER,
					CURLOPT_POSTFIELDS => json_encode($authHead),
				));
				$httpClientAuth->setRetry(3, 3);

				// Send login request
				$authData = $httpClientAuth->post('login');
				$authData = json_decode($authData, TRUE);
				// Auth client is no longer required because the api need to reinstance the client after authentication
				$httpClientAuth->close();
				
				if(!empty($authData['access_token']))
				{
					$token = trim($authData['access_token']);
					// Get token duration validity
					if(!empty($authData['expires_in']))
					{
						// Convert in minutes & keep some gap
						$cacheFile->setDuring(intval((intval($authData['expires_in']) - AUTH_TOKEN_VALIDITY_GAP) / 60));
					}
					// Set the cache file
					$cacheFile->writeData(json_encode(array('access_token' => $token)));
				}
				else
				{
					$status = 3;
					echo STR_STATUS[$status] . " => Failed to authenticate\n";

					if(isset($httpClientAuth) && is_a($httpClientAuth, 'HttpClient'))
					{
						$httpClientAuth->close();
					}
					if(isset($lockFile))
					{
						@unlink($lockFile);
					}

					exit($status);
				}
			}
			else
			{
				$status = 3;
				echo STR_STATUS[$status] . " => Failed to open auth file or missing parameters (" . $authFile  . ")\n";

				if(isset($httpClientAuth) && is_a($httpClientAuth, 'HttpClient'))
				{
					$httpClientAuth->close();
				}
				if(isset($lockFile))
				{
					@unlink($lockFile);
				}

				exit($status);
			}
		}

		// Instance httpClient for data fetch. Must be reinstanciated after authentication to avoid extremecloudiq API issue
		$httpClient = new ESIA\HTTP\HttpClient(API_URL);
		$httpClient->open();
		// Set header options for api get data
		$curlOptHttpHeader = DEFAULT_CURLOPT_HTTPHEADER;
		$curlOptHttpHeader[] = 'Authorization: Bearer ' . $token;
		$httpClient->setoptions(array(
			CURLOPT_CONNECTTIMEOUT => HTTP_CONNECTION_TIMEOUT,
			CURLOPT_TIMEOUT => HTTP_CONNECTION_TIMEOUT,
			CURLOPT_HTTPHEADER => $curlOptHttpHeader,
		));
		$httpClient->setRetry(3, 3);

		return $httpClient;
	}
	catch(CurlLibException | CurlException | Exception $e)
	{
		$status = 3;
		echo STR_STATUS[$status] . " => Cannot initialize API connection\n";
		ESIA\plugins\trace($isDebug, $e->getMessage());

		// Don't forget to close curl http clients if failed but client is initialized
		if(isset($httpClientAuth) && is_a($httpClientAuth, 'HttpClient'))
		{
			$httpClientAuth->close();
		}
		if(isset($httpClient) && is_a($httpClient, 'HttpClient'))
		{
			$httpClient->close();
		}

		if(isset($lockFile))
		{
			@unlink($lockFile);
		}

		exit($status);
	}
}

// Get perf data (cpu/mem) from cache file or API
function getPerfData($httpClient, $authFile, $cacheFileToken, $confName, $deviceId, $dataKey, $allowedDataAge, $isDebug)
{
	try
	{
		$cacheFilePerf = DEFAULT_CACHE_DIR_PATH . $confName . '_perf_' . $deviceId . '.json';
		$perfData = array();
		// Check for device perf (cpu/mem) cache
		$perfCache = new ESIA\CacheFile\CacheFile($cacheFilePerf);
		if($perfCache->isValid())
		{
			$perfData = json_decode($perfCache->getData(), TRUE);
		}

		if(!isset($perfData[$dataKey]))
		{
			$requestException = NULL;
			try
			{
				// Get data from API
				if(!isset($httpClient) || !is_a($httpClient, 'HttpClient') || $httpClient->getCurl() == NULL)
				{
					// Create http client if not yet availlable
					$httpClient = initHttpClient($authFile, $cacheFileToken, $isDebug);
				}

				// Get timestamp & interval in milliseconds
				$now = new DateTime();
				$now = $now->getTimestamp() * 1000;
				$interval = DEFAULT_TEST_INTERVAL * 1000;

				$params = array(
					'startTime' => $now - 18000000, // Take values on the last 5 hours or sometimes the API will return no data
					'endTime' => $now,
					'interval' => $interval,
				);
				$avgList = $httpClient->get('devices/' . $deviceId . '/history/cpu-mem', $params);
				$avgList = json_decode($avgList, TRUE);
				// Be sure to have the latest values
				$lastKey = count($avgList) - 1;
				if(!isset($avgList[$lastKey]['average_cpu']) || !isset($avgList[$lastKey]['average_memory']) || !isset($avgList[$lastKey]['timestamp']))
				{
					// At least one data is missing
					$lastKey--; // The latest value may be writting by another script. Take the previous latest
				}
				$perfData = $avgList[$lastKey];

				// Set during in minutes
				$perfCache->setDuring(CACHE_DEVICE_PERF_VALIDITY);
				// Set the cache file data
				$perfCache->writeData(json_encode($perfData));
			}
			catch(CurlLibException | CurlException | Exception $e)
			{
				// Save exception to display it if data from cache is to old than $allowedDataAge
				$requestException = $e;
				// Get old data from cache anyway
				$perfData = json_decode($perfCache->getData(), TRUE);
			}
		}

		// Check data age
		$minutesDiff = 0;
		if(isset($perfData['timestamp']))
		{
			$now = new DateTime();
			$minutesDiff = $now->getTimestamp() - ($perfData['timestamp'] / 1000);
			$minutesDiff = floor($minutesDiff / 60);
			if($minutesDiff > $allowedDataAge)
			{
				// Prevent that data is older than expected
				$errMsg = '';
				if(isset($requestException))
				{
					$errMsg = $requestException->getMessage() . "\n";
				}
				$errMsg .= "Data age: " . $minutesDiff . " minutes > " . $allowedDataAge;
				throw new Exception($errMsg);
			}
		}
		else
		{
			throw new Exception('Missing data timestamp');
		}

		if(isset($perfData[$dataKey]))
		{
			return array(
				'value' => $perfData[$dataKey],
				'age' => $minutesDiff,
			);
		}
		else
		{
			return NULL;
		}
	}
	catch(CurlLibException | CurlException | Exception $e)
	{
		$status = 3;
		echo STR_STATUS[$status] . " => Error fetching cpu-mem data\n";
		ESIA\plugins\trace($isDebug, $e->getMessage());

		exit($status);
	}
}

$status = 3;
$httpClient = NULL;

// Declare files used for this config
// Get config name from its file name to save different cache files matching the auth file name
// Get only file name from path
$confName = substr($authFile, (strrpos($authFile, '/') + 1));
// Remove file extension
$confName = substr($confName, 0, strrpos($confName, '.'));

// Lock file used to check if another process is refreshing the cache
$lockFile = DEFAULT_CACHE_DIR_PATH . $confName . '.lock';
// File used to save the auth token API
$cacheFileToken = DEFAULT_CACHE_DIR_PATH . $confName . '_token.json';
// File used to save devices data
$cacheFileDevices = DEFAULT_CACHE_DIR_PATH . $confName . '_devices.json';

// Check cache files dir or create it
if(!is_dir(DEFAULT_CACHE_DIR_PATH))
{
	// Create cache file dir
	if(!mkdir(DEFAULT_CACHE_DIR_PATH, 0755, TRUE)) // Create folder. TRUE => recursive
	{
		$status = 3;
		echo STR_STATUS[$status] . " => Failed to create cache file directory (" . DEFAULT_CACHE_DIR_PATH  . ")\n";
		exit($status);
	}
}

// Get devices data cache or refresh it from API
$devicesData = array(
	'total_count' => 0,
	'devices' => array(),
);
try
{
	// Init cache file data
	$devicesCache = new ESIA\CacheFile\CacheFile($cacheFileDevices);
	
	if($devicesCache->isValid())
	{
		$devicesData = json_decode($devicesCache->getData(), TRUE);
	}
	else
	{
		// Refresh devices data if not found in cache or expired & lock file is not used by another process
		// Check and re-create lock file
		$isCanRefresh = isLockFileReset($lockFile);

		if(!$isCanRefresh)
		{
			// Data cannot be refreshed (another process is using it). Use expired devices data to not block the script
			$devicesData = json_decode($devicesCache->getData(), TRUE);
		}
		else
		{
			// Refresh data
			// Create http client
			$httpClient = initHttpClient($authFile, $cacheFileToken, $isDebug, $lockFile);

			// Get all devices from api recursively because the api has a pagination where the limit is 100 devices max
			// Available views : BASIC, FULL, STATUS, LOCATION, CLIENT, DETAIL
			$params = array(
				'page' => 1,
				'limit' => 100,
				'views' => 'BASIC,CLIENT,LOCATION',
			);
			
			$hasMoreData = TRUE;
			while($hasMoreData)
			{
				$resp = $httpClient->get('devices', $params);
				$resp = @json_decode($resp, TRUE);

				if(!empty($resp['data']))
				{
					// Create an array where the key is the hostname so the next scripts using the cache file will find it faster in the list
					foreach($resp['data'] as $dev)
					{
						$devicesData['devices'][$dev['hostname']] = $dev;
					}

					// Get the total devices count
					if(isset($resp['total_count']))
					{
						$devicesData['total_count'] = $resp['total_count'];
					}

					if(isset($resp['count']) && $resp['count'] < $params['limit'])
					{
						// It was the last page
						$hasMoreData = FALSE;
					}
				}
				else
				{
					$hasMoreData = FALSE;
				}

				$params['page']++;
			}

			// Set during in minutes
			$devicesCache->setDuring(CACHE_DEVICES_VALIDITY);
			// Set the cache file data
			$devicesCache->writeData(json_encode($devicesData));

			// Delete lock file for safety if the lock file duration end after the device cache duration
			@unlink($lockFile);
		}
	}
}
catch(CurlLibException | CurlException | Exception $e)
{
	$status = 3;
	echo STR_STATUS[$status] . " => Error fetching devices list\n";
	ESIA\plugins\trace($isDebug, $e->getMessage());

	if(isset($httpClient) && is_a($httpClient, 'HttpClient'))
	{
		$httpClient->close();
	}

	if(isset($isCanRefresh) && $isCanRefresh == TRUE)
	{
		// Delete lock file for safety if the lock file duration end after the device cache duration
		@unlink($lockFile);
	}

	exit($status);
}

// Get device to test
if(isset($devicesData['devices'][$hostName]))
{
	$device = $devicesData['devices'][$hostName];
	$message = '';
	$rrd = '';

	// Test types
	if($type == 'cpu')
	{
		// CPU
		$data = getPerfData($httpClient, $authFile, $cacheFileToken, $confName, $device['id'], 'average_cpu', $allowedDataAge, $isDebug);

		if(isset($data['value']))
		{
			$cpu = intval($data['value']);
			$rrd = '|cpu=' . $cpu;
			if(isset($critical) && $cpu >= $critical)
			{
				$status = 2;
				$message = "CPU usage: $cpu% (>= $critical)";
			}
			elseif(isset($warning) && $cpu >= $warning)
			{
				$status = 1;
				$message = "CPU usage: $cpu% (>= $warning)";
			}
			else
			{
				$status = 0;
				$message = "CPU usage: $cpu%";
			}

			if(!empty($data['age']))
			{
				$message .= "\nData age: " . $data['age'] . " minutes";
			}

			// Write output
			echo STR_STATUS[$status] . " => " . $message . $rrd . "\n";
		}
		else
		{
			$status = 3;
			echo STR_STATUS[$status] . " => Cannot get cpu data\n";
		}
	}
	elseif($type == 'mem')
	{
		// Memory
		$data = getPerfData($httpClient, $authFile, $cacheFileToken, $confName, $device['id'], 'average_memory', $allowedDataAge, $isDebug);

		if(isset($data['value']))
		{
			$mem = intval($data['value']);
			$rrd = '|mem=' . $mem;
			if(isset($critical) && $mem >= $critical)
			{
				$status = 2;
				$message = "Memory usage: $mem% (>= $critical)";
			}
			elseif(isset($warning) && $mem >= $warning)
			{
				$status = 1;
				$message = "Memory usage: $mem% (>= $warning)";
			}
			else
			{
				$status = 0;
				$message = "Memory usage: $mem%";
			}

			if(!empty($data['age']))
			{
				$message .= "\nData age: " . $data['age'] . " minutes";
			}

			// Write output
			echo STR_STATUS[$status] . " => " . $message . $rrd . "\n";
		}
		else
		{
			$status = 3;
			echo STR_STATUS[$status] . " => Cannot get memory data\n";
		}
	}
	elseif($type == 'status')
	{
		// AP connected state
		if(empty($device['connected']))
		{
			$status = 2;
			$message = "$hostName is disconnected";
		}
		else
		{
			$status = 0;
			$message = "$hostName is connected";
		}
		// Write output
		echo STR_STATUS[$status] . " => " . $message . "\n";

		// Location info
		$locMessage = 'Not found';
		if(isset($device['locations']))
		{
			$locations = array();
			foreach($device['locations'] as $loc)
			{
				$locations[] = trim($loc['name']);
			}
			if(!empty($locations))
			{
				$locMessage = implode(' > ', $locations);
			}
		}
		echo "Device location: $locMessage\n";
	}
	elseif($type == 'config')
	{
		// Config mismatch status (updates, ...)
		if(!isset($device['config_mismatch']))
		{
			$status = 3;
			$message = "Configuration audit not found";
		}
		elseif(empty($device['config_mismatch']))
		{
			$status = 0;
			$message = "Configuration audit match";
		}
		else
		{
			// Mismatch found check for required status
			switch(strtolower($configMissState))
			{
				case 'c':
				case 'critical':
					$status = 2;
					break;
				case 'u':
				case 'unknown':
					$status = 3;
					break;
				case 'o':
				case 'ok':
					$status = 0;
					break;
				case 'w':
				case 'warning':
				default:
					$status = 1;
					break;
			}
			$message = "Configuration audit mismatch. Check updates and configuration issues.";
		}

		// Write output
		echo STR_STATUS[$status] . " => " . $message . "\n";
	}
	elseif($type == 'clients')
	{
		// Clients connected count
		$clients = 0;
		if(isset($device['active_clients']) && is_numeric($device['active_clients']))
		{
			$clients = intval($device['active_clients']);
		}

		$rrd = '|clients=' . $clients;
		if(isset($critical) && $clients >= $critical)
		{
			$status = 2;
			$message = "Clients connected: $clients (>= $critical)";
		}
		elseif(isset($warning) && $clients >= $warning)
		{
			$status = 1;
			$message = "Clients connected: $clients (>= $warning)";
		}
		else
		{
			$status = 0;
			$message = "Clients connected: $clients";
		}

		// Write output
		echo STR_STATUS[$status] . " => " . $message . $rrd . "\n";
	}
	elseif($type == 'inventory')
	{
		// Data for esia inventory webp
		$inv = array();
		// Device
		$inv[] = array(
			'name' => 'Extreme Networks - ' . $device['product_type'],
			'manufacturer' => 'Extreme Networks',
			'model' => $device['product_type'],
			'type' => 'CHASSIS',
			'serial' => $device['serial_number'],
			'hardwareVersion' => '',
			'firmwareVersion' => $device['software_version'],
		);
		// Interface
		$inv[] = array(
			'name' => 'ETH',
			'fullname' => '',
			'manufacturer' => 'Extreme Networks',
			'model' => '',
			'type' => 'NETINT',
			'hardwareVersion' => '',
			'firmwareVersion' => '',
			'option' => array(
				'macAdress' => $device['mac_address'],
				'IP' => $device['ip_address'],
			),
		);

		if($isJson)
		{
			echo json_encode($inv);
		}
		else
		{
			// Device
			echo 'name: ' . $inv[0]['name'] . ', manufacturer: ' . $inv[0]['manufacturer'] . ', model: ' . $inv[0]['model'] . ', type: ' . $inv[0]['type'] . ', serial: ' . $inv[0]['serial'] . ', hardwareVersion: ' . $inv[0]['hardwareVersion'] . ', firmwareVersion: ' . $inv[0]['firmwareVersion'] . "\n";
			// Interface
			echo 'name: ' . $inv[1]['name'] . ', fullname: ' . $inv[1]['fullname'] . ', manufacturer: ' . $inv[1]['manufacturer'] . ', model: ' . $inv[1]['model'] . ', type: ' . $inv[1]['type'] . ', hardwareVersion: ' . $inv[1]['hardwareVersion'] . ', firmwareVersion: ' . $inv[1]['firmwareVersion'] . ', macAdress: ' . $inv[1]['option']['macAdress'] . ', IP: ' . $inv[1]['option']['IP'] . "\n";
		}
		exit(0);
	}
	else
	{
		$status = 3;
		echo STR_STATUS[$status] . " => Unknown test type -T [cpu, mem, status, config, clients, inventory]\n";
	}
}
else
{
	$status = 3;
	echo STR_STATUS[$status] . " => Device not found (" . $hostName . ")\n";
}

// Don't forget to close curl http client
if(isset($httpClient) && is_a($httpClient, 'HttpClient'))
{
	$httpClient->close();
}

exit($status);
?>
