#!/usr/bin/perl -w
############################## check_snmp_draytek.pl ########
my $Version='1.0';
# Date : May 27 2024
# Author  : Nicolas Biersart
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use ESIA::SNMP;
use ESIA::PLUGIN;
use ESIA::CacheFile;
use JSON;


my $PROGNAME='check_snmp_draytek.pl';
my $PROGDESC = "Check draytek router SNMP";
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;
ESIA::PLUGIN::arm_end_signal($TIMEOUT);

my $return=0;
my ($opt_h,$opt_V, $json);


my $warning = 80;
my $critical = 90;
my $type = 'cpu';
my $oid_sysDescr = "1.3.6.1.2.1.1.1.0";
my $oid_proc_load = "1.3.6.1.2.1.25.3.3.1.2"; # %time the proc was not idle over last minute
my $oid_draytek_model='1.3.6.1.4.1.7367.3.1.0';
my $oid_draytek_firmware='1.3.6.1.4.1.7367.3.2.0';
my $oid_draytek_memory	= "1.3.6.1.4.1.7367.3.7.0";

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] -T <type> [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}

#
#MAIN
#
&Getopt::Long::config('bundling');
#on récupère les paramètres SNMP
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"T=s" => \$type,"type=s" => \$type,
	"w=s" => \$warning,"warning=s" => \$warning,
	"c=s" => \$critical,"critical=s" => \$critical,
	"J" => \$json,	 "json" => \$json,
);
# -h & --help print help
if ($opt_h || !defined($snmp))
{
	ESIA::SNMP::print_snmp_usage($PROGNAME,'-m namestorage');
	my @arr;
	push(@arr,["-w","--warning","warning threshold (default: $warning %)" ]);
	push(@arr,["-c","--critical","critical threshold (default: $critical %) "]);
	push(@arr,["-T","--type","Type of test (default: cpu)"]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}

#si l'host ne répond pas on blocke le script
my $Sysoid_value = ESIA::SNMP::getManufacturerOID($snmp);
if ($Sysoid_value ==-1)
{
    printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
    $snmp->close;
    exit $ERRORS{"UNKNOWN"};
}
if($type eq 'cpu')
{
	my $result = $snmp->get_table( $oid_proc_load);
	my $load=0;
	my $nbrcpu = 0;
	foreach my $key ( keys %$result)
	{
		$load=$load + $$result{$key};
		$nbrcpu++;
	}
	if($nbrcpu == 0)
	{
		print "UNKNOWN: No CPU information found\n";
		exit $ERRORS{'UNKNOWN'};
	}
	else
	{
		$load = $load / $nbrcpu;
		$load = sprintf("%.1f",$load);
		my $state = "OK";
		my $message = "< $warning%";
		if($load < $warning)
		{
			$state = "OK";
		}
		elsif($load < $critical)
		{
			$state = "WARNING";
			$message = "> $warning%";
		}
		else
		{
			$state = "CRITICAL";
			$message = "> $critical%";
		}
		printf("%s : CPU used %.1f%% %s|cpu=%.0f",$state,$load,$message,$load);
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
	}
}
elsif($type eq 'mem')
{
		my $state = "OK";
		my $message = "< $warning%";
		my $percent_used = ESIA::SNMP::get($snmp,"$oid_draytek_memory");
		if($percent_used < $warning)
		{
				$state = "OK";
		}
		elsif($percent_used < $critical)
		{
			$state = "WARNING";
			$message = "> $warning%";
		}
		else
		{
			$state = "CRITICAL";
			$message = "> $critical%";
		}
		printf("%s :memory used %.0f%% %s|mem=%.0f",$state,$percent_used,$message,$percent_used);
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
}
elsif( $type eq 'inventory' )
{
	my @array;
	my $sysdescr = ESIA::SNMP::get($snmp,"$oid_sysDescr");


	my $model = ESIA::SNMP::get($snmp,"$oid_draytek_model");
	my $firmware = ESIA::SNMP::get($snmp,"$oid_draytek_firmware");
	if($firmware=~/^([0-9]*\.[0-9]*\.[0-9]*)\sdrayos/)
	{
		$firmware=$1;
	}
	my $serial='';
	my %hash = ('name' => "DrayTek $model",'manufacturer'=>'DrayTek Corporation', 'model' => $model, 'type'=>'CHASSIS','serial'=>$serial,'hardwareVersion' =>'', 'firmwareVersion' => $firmware);
	push(@array, \%hash);

	my $output;
	if($json)
    {
       $output = encode_json(\@array);
    }
    else
    {
		foreach my $line(@array)
        {
			my $hash = \%$line;
			$output.= "name: ".$hash->{name}.", manufacturer: ".$hash->{manufacturer}.", model: ".$hash->{model}.", type: ".$hash->{type}.", serial: ".$hash->{serial};
			$output.=", hardwareVersion: ".$hash->{hardwareVersion}.", firmwareVersion: ".$hash->{firmwareVersion}."\n";
		}
    }
    print $output;
}
else
{
	print ("UNKNOWN: Incorrect or missed option -T, --type\n");
	$return = $ERRORS{'UNKNOWN'};
}

exit($return);
